/*Basic script for BitwigStudio - Maschine .. made by "Emerah"*/

loadAPI(1);
//defines the controller
host.defineController("Native Instruments", "Maschine", "1.1", "0D3ECCC0-B82F-11E3-A5E2-0800200C9A66");
//define the midi ports
host.defineMidiPorts(1, 1);
//auto-detect midi ports
host.addDeviceNameBasedDiscoveryPair(["Maschine MK2 Virtual Input"], ["Maschine MK2 Virtual Output"]);

function init() {
    host.getMidiInPort(0).setMidiCallback(onMidi);
    
    //Enable note input
    noteIn = host.getMidiInPort(0).createNoteInput("Maschine Pads", "80????", "90????");
    application = host.createApplication();
    transport = host.createTransport();
    device = host.createCursorDevice();
    track = host.createCursorTrack(1, 0);
    arranger = host.createArranger(1);
    
    //set observers for bi-directional communication
    transport.addIsPlayingObserver(function(isPlaying) {
        sendChannelController(0, 0, isPlaying ? 127 : 0);
    });
    transport.addIsRecordingObserver(function(isRecording) {
        sendChannelController(0, 2, isRecording ? 127 : 0);
    });
    transport.addIsLoopActiveObserver(function(isLooping){
        sendChannelController(0, 3, isLooping ? 127 : 0);
    });
    transport.addPunchInObserver(function(isPunchIn){
        sendChannelController(0, 4, isPunchIn ? 127 : 0); 
    });
    transport.addPunchOutObserver(function(isPunchOut){
        sendChannelController(0, 5, isPunchOut ? 127 : 0); 
    });
    transport.addClickObserver(function(isClickEnabled){
        sendChannelController(0, 6, isClickEnabled ? 127 : 0);
    });
    track.getMute().addValueObserver(function(isMuted){
        sendChannelController(0, 10, isMuted ? 127 : 0);
    });
    track.getSolo().addValueObserver(function(isSoloed){
        sendChannelController(0, 11, isSoloed ? 127 : 0);
    });
    track.getArm().addValueObserver(function(isArmed){
        sendChannelController(0, 12, isArmed ? 127 : 0);
    });
    track.getVolume().addValueObserver(128, function(value) {
        sendChannelController(0, 13, value); 
    });
    track.getPan().addValueObserver(128, function(value) {
        sendChannelController(0, 14, value); 
    });
    arranger.addPlaybackFollowObserver(function(isFollowing) {
        sendChannelController(0, 38, isFollowing ? 127 : 0);    
    });
    arranger.addTrackRowHeightObserver(function(isFolded) {
        sendChannelController(0, 39, isFolded ? 0 : 127);    
    });
    
    primaryInstrument = track.getPrimaryInstrument();
    for ( var param = 0; param < 8; param++) {
            var macro = primaryInstrument.getMacro(param).getAmount();
            macro.setIndication(true);
            var parameter = device.getParameter(param);
            parameter.setIndication(true);
            parameter.setLabel("Param" + (param + 1));
	}
}

function onMidi(status, data1, data2) {
    
    //Transport controls
    if (isChannelController(status) && data1 == 0 && data2 > 0) {
        transport.play();
    }
    
    if (isChannelController(status) && data1 == 1 && data2 > 0) {
        transport.stop();
    }
    
    if (isChannelController(status) && data1 == 2 && data2 > 0) {
        transport.record();
    }
    
    if (isChannelController(status) && data1 == 3 && data2 > 0) {
        transport.toggleLoop();
    }
    
    if (isChannelController(status) && data1 == 4 && data2 > 0) {
        transport.togglePunchIn();
    }
    
    if (isChannelController(status) && data1 == 5 && data2 > 0) {
        transport.togglePunchOut();
    }
    
    if (isChannelController(status) && data1 == 6 && data2 > 0) {
        transport.toggleClick();
    }
    
    //if (isChannelController(status) && data1 == 7 && data2 >= 0) {
    //    transport.getTempo().set(data2, 180);
    //}
        
    //Track controls
    if (isChannelController(status) && data1 == 8 && data2 > 0) {
        track.selectPrevious();
    }
    
    if (isChannelController(status) && data1 == 9 && data2 > 0) {
        track.selectNext();
    }
    
    if (isChannelController(status) && data1 == 10 && data2 > 0) {
        track.getMute().toggle();
    }
    
    if (isChannelController(status) && data1 == 11 && data2 > 0) {
        track.getSolo().toggle();
    }
    
    if (isChannelController(status) && data1 == 12 && data2 > 0) {
        track.getArm().toggle();
    }
    
    if (isChannelController(status) && data1 == 13 && data2 >= 0) {
        track.getVolume().set(data2, 128);
    }
    
    if (isChannelController(status) && data1 == 14 && data2 >= 0) {
        track.getPan().set(data2, 128);
    }
    
    //Device controls
    
    if (isChannelController(status) && data1 >= 15 && data1 <= 22 && data2 >= 0) {
        device.getParameter(data1 - 15).set(data2, 128);    
    }
    
    if (isChannelController(status) && data1 >= 40 && data1 <= 47 && data2 >= 0) {
        device.getMacro(data1 - 40).getAmount().set(data2, 128);    
    }
    
    if (isChannelController(status) && data1 == 23 && data2 > 0) {
        device.previousParameterPage();
    }
    
    if (isChannelController(status) && data1 == 24 && data2 > 0) {
        device.nextParameterPage();
    }
    
    if (isChannelController(status) && data1 == 25 && data2 > 0) {
        device.switchToPreviousPreset();
    }
    
    if (isChannelController(status) && data1 == 26 && data2 > 0) {
        device.switchToNextPreset();
    }
    
    if (isChannelController(status) && data1 == 27 && data2 > 0) {
        device.selectPrevious();
    }
    
    if (isChannelController(status) && data1 == 28 && data2 > 0) {
        device.selectNext();
    }
    
    if (isChannelController(status) && data1 == 29 && data2 > 0) {
        device.toggleEnabledState();
    }
    
    //Application views controls
    if (isChannelController(status) && data1 == 30 && data2 > 0) {
        application.nextPerspective();
    }
    
    if (isChannelController(status) && data1 == 31 && data2 > 0) {
        application.toggleBrowserVisibility();
    }
    
    if (isChannelController(status) && data1 == 32 && data2 > 0) {
        application.undo();
    }
    
    if (isChannelController(status) && data1 == 33 && data2 > 0) {
        application.redo();
    }
    
    if (isChannelController(status) && data1 == 34 && data2 > 0) {
        application.toggleMixer();
    }
    
    if (isChannelController(status) && data1 == 35 && data2 > 0) {
        application.toggleDevices();
    }
    
    if (isChannelController(status) && data1 == 36 && data2 > 0) {
        application.toggleNoteEditor();
    }
    
    if (isChannelController(status) && data1 == 37 && data2 > 0) {
        application.toggleAutomationEditor();
    }
    
    //Arranger controls
    if (isChannelController(status) && data1 == 38 && data2 > 0) {
        arranger.togglePlaybackFollow();
    }
    
    if (isChannelController(status) && data1 == 39 && data2 > 0) {
        arranger.toggleTrackRowHeight();
    }
}

function exit() {
    
}

